from Products.CMFCore import utils as cmf_utils
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.permissions import AddPortalFolders
from Products.CMFDynamicViewFTI import content_for_tests
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation


def initialize(context):
    # (DynamicViewTypeInformation factory is created from ZCML)
    cmf_utils.registerIcon(DynamicViewTypeInformation, "images/typeinfo.gif", globals())

    context.registerClass(
        content_for_tests.DynFolder,
        permission=AddPortalFolders,
        constructors=(("addDynFolder", content_for_tests.addDynFolder),),
        icon="images/typeinfo.gif",
    )

    context.registerClass(
        content_for_tests.DynDocument,
        permission=AddPortalContent,
        constructors=(("addDynDocument", content_for_tests.addDynDocument),),
        icon="images/typeinfo.gif",
    )



from plone.app import testing
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.base.utils import unrestricted_construct_instance
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup import EXTENSION
from Products.GenericSetup import profile_registry

import transaction
import unittest


class CMFDynamicViewFTIFixture(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        profile_registry.registerProfile(
            "CMFDVFTI_sampletypes",
            "CMFDynamicViewFTI Sample Content Types",
            "Extension profile including CMFDVFTI sample content types",
            "profiles/sample_types",
            "CMFDynamicViewFTI",
            EXTENSION,
            for_=ISiteRoot,
        )
        import Products.CMFDynamicViewFTI.tests

        self.loadZCML(
            name="browserdefault.zcml", package=Products.CMFDynamicViewFTI.tests
        )

        from plone.testing.zope import _INSTALLED_PRODUCTS

        if "Products.CMFDynamicViewFTI" not in _INSTALLED_PRODUCTS.keys():
            # replicate plone.testing.zope.installProduct
            # when running the tests via tox Products.CMFDynamicViewFTI
            # folder location is different than the other Products.*
            # packages.
            # `installProduct` checks for that and bails out otherwise.
            from AccessControl.class_init import InitializeClass
            from OFS.Application import get_folder_permissions
            from OFS.Application import install_product
            from OFS.Folder import Folder
            from pathlib import Path

            import inspect

            module_path = Path(inspect.getfile(Products.CMFDynamicViewFTI))
            product_folder = str(module_path.parent.parent)

            install_product(
                app,
                product_folder,
                "CMFDynamicViewFTI",
                [],
                get_folder_permissions(),
                raise_exc=1,
            )
            InitializeClass(Folder)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, "CMFDynamicViewFTI:CMFDVFTI_sampletypes")


CDV_FIXTURE = CMFDynamicViewFTIFixture()
CDV_FUNCTIONAL_TESTING = testing.FunctionalTesting(
    bases=(CDV_FIXTURE,), name="CMFDynamicViewFTI Testing:Functional"
)


class CMFDVFTITestCase(unittest.TestCase):
    """This is a stub now, but in case you want to try
    something fancy on Your Branch (tm), put it here.
    """

    layer = CDV_FUNCTIONAL_TESTING

    def setUp(self):
        """Set up before each test."""
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        unrestricted_construct_instance("DynFolder", self.portal, id="folder")
        self.folder = self.portal.folder
        self.types = getToolByName(self.portal, "portal_types")
        self.fti = self.types["DynFolder"]
        transaction.commit()



from Products.CMFCore.interfaces import ITypeInformation
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from Products.CMFDynamicViewFTI.tests import CMFDVFTITestCase
from zope.interface.verify import verifyObject

import transaction


fti_meta_type = DynamicViewTypeInformation.meta_type


class TestFTI(CMFDVFTITestCase.CMFDVFTITestCase):
    def _makeOne(self):
        # Create and return a DynFolder
        self.folder.invokeFactory("DynFolder", id="dynfolder")
        return self.folder.dynfolder

    def test_doesImplementITypeInformation(self):
        iface = ITypeInformation
        self.assertTrue(iface.providedBy(self.fti))
        self.assertTrue(verifyObject(iface, self.fti))

    def test_doesImplementIDynamicViewTypeInformation(self):
        iface = IDynamicViewTypeInformation
        self.assertTrue(iface.providedBy(self.fti))
        self.assertTrue(verifyObject(iface, self.fti))

    def test_meta_type(self):
        self.assertEqual(self.fti.meta_type, fti_meta_type)

    def test_paranoid_subclass_test(self):
        self.assertTrue(isinstance(self.fti, DynamicViewTypeInformation))

    def test_CreateDynFolder(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getPortalTypeName(), "DynFolder")
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getId(), "DynFolder")
        self.assertEqual(info.Title(), "DynFolder")
        self.assertEqual(info.getDefaultViewMethod(dynfolder), "index_html")
        self.assertEqual(
            info.getAvailableViewMethods(dynfolder),
            ("index_html", "custom_view", "zope3_view"),
        )

    def test_DynFolderDefaultView(self):
        dynfolder = self._makeOne()
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "index_html")

    def test_DynFolderCustomView(self):
        dynfolder = self._makeOne()
        self.types.DynFolder.manage_changeProperties(default_view="custom_view")
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "custom_view")

    def test_DynFolderViewFromLayout(self):
        dynfolder = self._makeOne()
        dynfolder.layout = "custom_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "custom_view")

    def test_DynFolderViewFromCallableLayout(self):
        dynfolder = self._makeOne()
        dynfolder.layout = lambda: "custom_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "custom_view")

    def test_NoneLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = None
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "index_html")

    def test_EmptyLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = ""
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), "index_html")

    def test_InvalidLayoutRaisesTypeError(self):
        dynfolder = self._makeOne()
        dynfolder.layout = object()
        info = self.types.getTypeInfo(dynfolder)
        self.assertRaises(TypeError, info.getViewMethod, dynfolder)

    def test_EnforceLayoutAvailable(self):
        dynfolder = self._makeOne()
        dynfolder.layout = "custom_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, enforce_available=True), "custom_view"
        )

    def test_UnavailableLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = "bad_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, enforce_available=True), "index_html"
        )

    def test_CheckLayoutExists(self):
        dynfolder = self._makeOne()
        dynfolder.manage_addDTMLMethod("custom_view", file="")
        dynfolder.layout = "custom_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, check_exists=True), "custom_view"
        )

    def test_MissingLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = "bad_view"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder, check_exists=True), "index_html")

    def test_DynFolderDefaultPage(self):
        dynfolder = self._makeOne()
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_DynFolderDefaultPageFromAttribute(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = "custom_page"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), "custom_page")

    def test_DynFolderDefaultPageFromCallable(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = lambda: "custom_page"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), "custom_page")

    def test_NoneDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = None
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_EmptyDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = ""
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_InvalidDefaultPageRaisesTypeError(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = object()
        info = self.types.getTypeInfo(dynfolder)
        self.assertRaises(TypeError, info.getDefaultPage, dynfolder)

    def test_CheckDefaultPageExists(self):
        dynfolder = self._makeOne()
        dynfolder.manage_addDTMLMethod("custom_page", file="")
        dynfolder.default_page = "custom_page"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getDefaultPage(dynfolder, check_exists=True), "custom_page"
        )

    def test_MissingDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = "bad_page"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder, check_exists=True), None)

    def test_NonFolderishObjectReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.isPrincipiaFolderish = 0
        dynfolder.default_page = "custom_page"
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_DefaultViewWithBadLayoutUseFallback(self):
        dynfolder = self._makeOne()
        dynfolder.layout = "bad_view"
        info = self.types.getTypeInfo(dynfolder)
        # fallback not returned if not activated
        self.assertFalse(info.default_view_fallback)
        self.assertEqual(info.defaultView(dynfolder), "bad_view")
        # enable fallback
        info.default_view_fallback = True
        self.assertEqual(info.defaultView(dynfolder), info.default_view)


class TestModifyDefaultPage(CMFDVFTITestCase.CMFDVFTITestCase):
    def _makeOne(self):
        # Create and return a DynFolder
        self.folder.invokeFactory("DynFolder", id="dynfolder")
        dynfolder = self.folder.dynfolder
        dynfolder.invokeFactory("DynDocument", id="default_document")
        dynfolder.setDefaultPage("default_document")
        return dynfolder

    def test_rename_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), "default_document")
        transaction.commit()
        dynfolder.manage_renameObject("default_document", "renamed_default")
        self.assertFalse("default_document" in dynfolder.objectIds())
        self.assertTrue("renamed_default" in dynfolder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), "renamed_default")
        self.assertEqual(dynfolder.getProperty("default_page", ""), "renamed_default")

    def test_delete_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), "default_document")
        dynfolder.manage_delObjects(["default_document"])
        self.assertFalse("default_document" in dynfolder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), None)
        self.assertEqual(dynfolder.getProperty("default_page", ""), "")

    def test_cut_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), "default_document")
        transaction.commit()
        clipboard = dynfolder.manage_cutObjects(["default_document"])
        self.folder.manage_pasteObjects(clipboard)
        self.assertFalse("default_document" in dynfolder.objectIds())
        self.assertTrue("default_document" in self.folder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), None)
        self.assertEqual(dynfolder.getProperty("default_page", ""), "")



from setuptools import find_packages
from setuptools import setup


version = "7.0.1"

long_description = open("README.rst").read()
long_description += "\n"
long_description += open("CHANGES.rst").read()


setup(
    name="Products.CMFDynamicViewFTI",
    version=version,
    description="CMFDynamicViewFTI is a product for dynamic views in CMF.",
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: Zope Public License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="Zope CMF Plone dynamic view",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/Products.CMFDynamicViewFTI",
    license="ZPL",
    packages=find_packages(),
    namespace_packages=["Products"],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            "Products.GenericSetup",
            "plone.base",
            "plone.app.testing >= 4.2.5",
            "plone.testing",
            "zope.publisher",
        ]
    ),
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "zope.browser",
        "zope.browsermenu",
        "Products.CMFCore",
        "ExtensionClass",
        "Zope",
    ],
)
